@@ -93,10 +93,7 @@ impl StdioOrRid {
   ) -> Result<std::process::Stdio, AnyError> {
     match &self {
       StdioOrRid::Stdio(val) => Ok(val.as_stdio()),
-      StdioOrRid::Rid(rid) => {
-        let file = StdFileResource::clone_file(state, *rid)?;
-        Ok(file.into())
-      }
+      StdioOrRid::Rid(rid) => StdFileResource::as_stdio(state, *rid),
     }
   }
 }
