@@ -8,7 +8,6 @@ use deno_core::ModuleSpecifier;
 pub struct BootstrapOptions {
   /// Sets `Deno.args` in JS runtime.
   pub args: Vec<String>,
-  pub apply_source_maps: bool,
   pub cpu_count: usize,
   pub debug_flag: bool,
   pub enable_testing_features: bool,
@@ -28,7 +27,6 @@ impl BootstrapOptions {
     let payload = json!({
       // Shared bootstrap args
       "args": self.args,
-      "applySourceMaps": self.apply_source_maps,
       "cpuCount": self.cpu_count,
       "debugFlag": self.debug_flag,
       "denoVersion": self.runtime_version,
